﻿using System.Collections.ObjectModel;
using Windows.Foundation;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;
using Syncfusion.UI.Xaml.Diagram;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml.Shapes;
using System.Runtime.Serialization;
using Syncfusion.UI.Xaml.Diagram.Controller;

namespace DiagramBuilder.ViewModel
{
    public class ConnectorVM : GroupableVM, IConnectorVM
    {
        public ConnectorVM(IUndoRedoController undoRedoController):this()
        {
            UndoRedoController = undoRedoController;
        }

        public ConnectorVM()
        {
            Stroke = new SolidColorBrush(new Color() { A = 0xFF, R = 0x7F, G = 0x7F, B = 0x7F });
        }

        #region IConnector

        object _mSourceNode = null;
        public object SourceNode
        {
            get
            {
                return _mSourceNode;
            }
            set
            {
                if (_mSourceNode == null || !_mSourceNode.Equals(value))
                {
                    _mSourceNode = value;
                    OnPropertyChanged(ConnectorConstants.SourceNode);
                }
            }
        }


        object _mTargetNode = null;
        public object TargetNode
        {
            get
            {
                return _mTargetNode;
            }
            set
            {
                if (_mTargetNode == null || !_mTargetNode.Equals(value))
                {
                    _mTargetNode = value;
                    OnPropertyChanged(ConnectorConstants.TargetNode);
                }
            }
        }


        IPort _mSourcePort = null;
        public IPort SourcePort
        {
            get
            {
                return _mSourcePort;
            }
            set
            {
                if (_mSourcePort != value)
                {
                    _mSourcePort = value;
                    OnPropertyChanged(ConnectorConstants.SourcePort);
                }
            }
        }


        IPort _mTargetPort = null;
        public IPort TargetPort
        {
            get
            {
                return _mTargetPort;
            }
            set
            {
                if (_mTargetPort != value)
                {
                    _mTargetPort = value;
                    OnPropertyChanged(ConnectorConstants.TargetPort);
                }
            }
        }


        IList<IConnectorSegment> _mSegments = null;
        public IList<IConnectorSegment> Segments
        {
            get
            {
                return _mSegments;
            }
            set
            {
                if (_mSegments != value)
                {
                    _mSegments = value;
                    OnPropertyChanged(ConnectorConstants.Segments);
                }
            }
        }


        Point _mSourcePoint = new Point(0, 0);
        public Point SourcePoint
        {
            get
            {
                return _mSourcePoint;
            }
            set
            {
                if (_mSourcePoint != value)
                {
                    _mSourcePoint = value;
                    OnPropertyChanged(ConnectorConstants.SourcePoint);
                }
            }
        }


        Point _mTargetPoint = new Point(0, 0);
        public Point TargetPoint
        {
            get
            {
                return _mTargetPoint;
            }
            set
            {
                if (_mTargetPoint != value)
                {
                    _mTargetPoint = value;
                    OnPropertyChanged(ConnectorConstants.TargetPoint);
                }
            }
        }


        Style _mConnectorGeometryStyle = null;
        public Style ConnectorGeometryStyle
        {
            get
            {
                return _mConnectorGeometryStyle;
            }
            set
            {
                if (_mConnectorGeometryStyle != value)
                {
                    _mConnectorGeometryStyle = value;
                    OnPropertyChanged(ConnectorConstants.ConnectorGeometryStyle);
                }
            }
        }


        object _mSourceDecorator = null;
        public object SourceDecorator
        {
            get
            {
                return _mSourceDecorator;
            }
            set
            {
                if (_mSourceDecorator != value)
                {
                    _mSourceDecorator = value;
                    OnPropertyChanged(ConnectorConstants.SourceDecorator);
                }
            }
        }


        object _mTargetDecorator = null;
        public object TargetDecorator
        {
            get
            {
                return _mTargetDecorator;
            }
            set
            {
                if (_mTargetDecorator != value)
                {
                    _mTargetDecorator = value;
                    OnPropertyChanged(ConnectorConstants.TargetDecorator);
                }
            }
        }


        Style _mSourceDecoratorStyle = null;
        public Style SourceDecoratorStyle
        {
            get
            {
                return _mSourceDecoratorStyle;
            }
            set
            {
                if (_mSourceDecoratorStyle != value)
                {
                    _mSourceDecoratorStyle = value;
                    OnPropertyChanged(ConnectorConstants.SourceDecoratorStyle);
                }
            }
        }


        Style _mTargetDecoratorStyle = null;
        public Style TargetDecoratorStyle
        {
            get
            {
                return _mTargetDecoratorStyle;
            }
            set
            {
                if (_mTargetDecoratorStyle != value)
                {
                    _mTargetDecoratorStyle = value;
                    OnPropertyChanged(ConnectorConstants.TargetDecoratorStyle);
                }
            }
        }


        ConnectorConstraints _mConstraints = ConnectorConstraints.Default;
        public ConnectorConstraints Constraints
        {
            get
            {
                return _mConstraints;
            }
            set
            {
                if (_mConstraints != value)
                {
                    _mConstraints = value;
                    OnPropertyChanged(ConnectorConstants.Constraints);
                }
            }
        }


        BezierSmoothness _mBezierSmoothness = BezierSmoothness.None;
        public BezierSmoothness BezierSmoothness
        {
            get
            {
                return _mBezierSmoothness;
            }
            set
            {
                if (_mBezierSmoothness != value)
                {
                    _mBezierSmoothness = value;
                    OnPropertyChanged(ConnectorConstants.BezierSmoothness);
                }
            }
        }


        double _mBridgeSpace = 15d;
        public double BridgeSpace
        {
            get
            {
                return _mBridgeSpace;
            }
            set
            {
                if (_mBridgeSpace != value)
                {
                    _mBridgeSpace = value;
                    OnPropertyChanged(ConnectorConstants.BridgeSpace);
                }
            }
        }
        #endregion

        #region IConnectorVM
        List<object> _mCaps = null;
        public List<object> Caps
        {
            get
            {
                return _mCaps;
            }
            set
            {
                if (_mCaps != value)
                {
                    _mCaps = value;
                    OnPropertyChanged(ConnectorConstants.Caps);
                }
            }
        }

        string _mSourceCap = "M0,0 z";
        [DataMember]
        public string SourceCap
        {
            get
            {
                return _mSourceCap;
            }
            set
            {
                if (_mSourceCap != value)
                {
                    _mSourceCap = value;
                    OnPropertyChanged(ConnectorConstants.SourceCap);
                }
            }
        }

        string _mTargetCap = "M0,0 L10,5 L0,10 L 0,0";
        [DataMember]
        public string TargetCap
        {
            get
            {
                return _mTargetCap;
            }
            set
            {
                if (_mTargetCap != value)
                {
                    _mTargetCap = value;
                    OnPropertyChanged(ConnectorConstants.TargetCap);
                }
            }
        }

        ConnectType _mType = ConnectType.Orthogonal;
        [DataMember]
        public ConnectType Type
        {
            get
            {
                return _mType;
            }
            set
            {
                if (_mType != value)
                {
                    _mType = value;
                    OnPropertyChanged(ConnectorConstants.Type);
                }
            }
        }
        #endregion

        protected override void OnPropertyChanged(string name)
        {
            DoLog(name);
            base.OnPropertyChanged(name);
            switch (name)
            {
                case ConnectorConstants.SourceCap:
                    OnSourceCapChanged();
                    break;
                case ConnectorConstants.TargetCap:
                    OnTargetCapChanged();
                    break;
                case ConnectorConstants.Type:
                    OnTypeChanged();
                    break;
            }
        }

        private void OnTypeChanged()
        {
            switch (Type)
            {
                case ConnectType.Bezier:
                    Segments = new ObservableCollection<IConnectorSegment> {new CubicCurveSegment()};
                    break;
                case ConnectType.Orthogonal:
                    Segments = new ObservableCollection<IConnectorSegment> {new OrthogonalSegment()};
                    break;
                case ConnectType.Straight:
                    Segments = new ObservableCollection<IConnectorSegment> {new StraightSegment()};
                    break;
            }
        }

        private void OnTargetCapChanged()
        {
            TargetDecorator = TargetCap;
        }

        private void OnSourceCapChanged()
        {
            SourceDecorator = SourceCap;
        }

        protected override void ApplyStyle(Style style)
        {
            ConnectorGeometryStyle = style;

            SourceDecoratorStyle = getDecoratorStyle();
            TargetDecoratorStyle = getDecoratorStyle();
        }

        private Style getDecoratorStyle()
        {
            Style decoratorStyle = GetCustomStyle();
            decoratorStyle.Setters.Add(new Setter(Path.StretchProperty, Stretch.Fill));
            decoratorStyle.Setters.Add(new Setter(Path.WidthProperty, 10));
            decoratorStyle.Setters.Add(new Setter(Path.HeightProperty, 10));
            if (Stroke != null)
            {
                decoratorStyle.Setters.Add(new Setter(Path.FillProperty, Stroke));
            }
            return decoratorStyle;
        }

        #region UndoRedo

        public override object GetData()
        {
            return new[]
            {
                new ConnectorChangeParameters()
                {
                    _sourceCap = SourceCap,
                    _targetCap = TargetCap,
                    _type = Type
                },
                base.GetData()
            };
        } 

         public override IHistoryParameter Undo(IHistoryParameter param)
        {
            if (param.Action is Array)
            {
                return (IHistoryParameter)Revert(param);
            }
            return base.Redo(param);
        }

        public override IHistoryParameter Redo(IHistoryParameter param)
        {
            if (param.Action is Array)
            {
                return (IHistoryParameter)Revert(param);
            }
            return base.Redo(param);
        }

        public override object Revert(object param)
        {
            var parameter = (IHistoryParameter) param;
            if (parameter.Action is Array)
            {
                var current = GetData();
                ConnectorChangeParameters tostate = (ConnectorChangeParameters) (parameter.Action as object[])[0];
                if (SourceCap != tostate._sourceCap)
                {
                    SourceCap = tostate._sourceCap;
                }
                if (TargetCap != tostate._targetCap)
                {
                    TargetCap = tostate._targetCap;
                }
                if (Type != tostate._type)
                {
                    Type = tostate._type;
                }
                base.Revert((parameter.Action as object[])[1]);
                return new HistoryParameter()
                {
                    Name = "Connector",
                    Action = current
                };
            }
            return base.Revert(parameter.Action);
        }

        public override bool CheckLog(string name)
        {
            switch (name)
            {
                case ConnectorConstants.SourceCap:
                case ConnectorConstants.TargetCap:
                case ConnectorConstants.Type:
                    return true;
            }
            return base.CheckLog(name);
        }

        public override void SetData()
        {
            CurrentState = GetData();
        }

        #endregion
    }

    public interface IConnectorVM : IGroupableVM, IConnector
    {
        List<object> Caps { get; set; }
        string SourceCap { get; set; }
        string TargetCap { get; set; }
        ConnectType Type { get; set; }
    }

    internal static class ConnectorConstants
    {
        public const string SourceNode = "SourceNode";
        public const string TargetNode = "TargetNode";
        public const string SourcePort = "SourcePort";
        public const string TargetPort = "TargetPort";
        public const string Segments = "Segments";
        public const string SourcePoint = "SourcePoint";
        public const string TargetPoint = "TargetPoint";
        public const string ConnectorGeometryStyle = "ConnectorGeometryStyle";
        public const string SourceDecorator = "SourceDecorator";
        public const string TargetDecorator = "TargetDecorator";
        public const string SourceDecoratorStyle = "SourceDecoratorStyle";
        public const string TargetDecoratorStyle = "TargetDecoratorStyle";
        public const string AutoBind = "AutoBind";
        public const string ID = "ID";
        public const string Key = "Key";
        public const string IsSelected = "IsSelected";
        public const string ZIndex = "ZIndex";
        public const string BridgeSpace = "BridgeSpace";
        public const string Annotations = "Annotations";
        public const string Constraints = "Constraints";
        public const string BezierSmoothness = "BezierSmoothness";
        public const string ParentGroup = "ParentGroup";

        public const string Caps = "Caps";
        public const string SourceCap = "SourceCap";
        public const string TargetCap = "TargetCap";
        public const string Type = "Type";
        public const string Types = "Types";
    }
}
